uniform float time;
uniform vec2 resolution;
uniform float boltsCount;
uniform float boltSpeed;
uniform float intensity;

float hash(float n) {
    return fract(sin(n * 78.233) * 43758.5453);
}

vec2 bolt(vec2 uv, float seed) {
    float t = time * boltSpeed;
    float result = 0.0;
    float glow = 0.0;
    float appear = step(0.7, hash(floor(t * 2.0) + seed * 43.31));

    float xStart = hash(seed * 13.37) * 2.0 - 1.0;
    float boltProgress = fract(t * 3.0 + seed * 42.7) * 1.5;
    float boltIntensity = 1.0 - smoothstep(0.0, 0.3, abs(boltProgress - 0.8));

    vec2 pos = vec2(xStart, 1.2);
    float prevX = xStart;

    for (int i = 0; i < 15; i++) {
        float fi = float(i);
        float segmentT = t * 20.0 + fi * 17.8 + seed * 44.65;

        float nextX = prevX + (hash(segmentT) * 2.0 - 1.0) * (0.2 + 0.2 * hash(fi + seed * 71.3));
        float nextY = 1.2 - (fi + 1.0) * 0.15;

        float variation = 0.8 + 0.4 * sin(t * 20.0 + fi + seed * 13.7);
        float lineWidth = 0.01 * (1.0 - fi/20.0) * variation;

        vec2 a = pos;
        vec2 b = vec2(nextX, nextY);

        vec2 pa = uv - a;
        vec2 ba = b - a;
        float h = clamp(dot(pa, ba) / dot(ba, ba), 0.0, 1.0);
        float dist = length(pa - ba * h);

        float progress = smoothstep(0.0, 1.0, (1.0 - nextY) / 2.0);
        float progressMask = step(progress, boltProgress);

        float segment = smoothstep(lineWidth, 0.0, dist) * progressMask;
        float segmentGlow = exp(-dist * dist * 100.0) * progressMask;

        result = max(result, segment);
        glow = max(glow, segmentGlow);

        if (i > 2 && i < 10 && hash(fi * 37.7 + seed) > 0.7) {
            vec2 branchPos = a + ba * hash(fi * 32.5 + seed);
            float branchAngle = 0.5 + hash(fi + seed * 17.3);
            float branchLength = 0.1 + 0.2 * hash(fi * 2.3 + seed);

            vec2 branchEnd = branchPos + vec2(
                cos(branchAngle) * branchLength * (hash(fi) * 2.0 - 1.0),
                -sin(branchAngle) * branchLength
            );

            vec2 bpa = uv - branchPos;
            vec2 bba = branchEnd - branchPos;
            float bh = clamp(dot(bpa, bba) / dot(bba, bba), 0.0, 1.0);
            float bdist = length(bpa - bba * bh);

            float bsegment = smoothstep(lineWidth * 0.7, 0.0, bdist) * progressMask;
            float bsegmentGlow = exp(-bdist * bdist * 80.0) * progressMask;

            result = max(result, bsegment * 0.7);
            glow = max(glow, bsegmentGlow * 0.5);
        }

        prevX = nextX;
        pos = b;
    }

    float flicker = 0.7 + 0.3 * sin(t * 40.0 + seed * 76.2);
    return vec2(result, glow * 0.6) * flicker * appear * boltIntensity;
}

void main() {
    vec2 uv = (gl_FragCoord.xy * 2.0 - resolution.xy) / resolution.y;
    vec3 color = vec3(0.0);

    for(float i = 0.0; i < boltsCount; i++) {
        float seed = i + 1.0;
        vec2 b = bolt(uv, seed);

        float coreBrightness = b.x;
        float glowBrightness = b.y;

        float colorVar = hash(seed * 43.2);
        vec3 coreColor = mix(
            vec3(0.7, 0.9, 1.0),
            vec3(1.0),
            coreBrightness
        );

        vec3 glowColor = mix(
            vec3(0.1, 0.4, 0.8),
            vec3(0.5, 0.7, 1.0),
            glowBrightness
        );

        color += coreBrightness * coreColor * intensity * 2.0;
        color += glowBrightness * glowColor * intensity;
    }

    color = 1.0 - exp(-color * 1.7);

    gl_FragColor = vec4(color, 1.0);
}
